@@ -1,9 +1,12 @@
 import os
 import getopt
 import sys
-import dbus, dbus.service, dbus.glib
+import dbus, dbus.service, dbus.mainloop.glib
 import gobject
 
+dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
+dbus.mainloop.glib.threads_init()
+
 import logging
 log = logging.getLogger("Main")
 
@@ -15,6 +18,7 @@ from conduit.TypeConverter import TypeConverter
 from conduit.SyncSet import SyncSet
 from conduit.Synchronization import SyncManager
 from conduit.DBus import DBusInterface
+from conduit.Settings import Settings
 
 APPLICATION_DBUS_IFACE="org.conduit.Application"
 
@@ -43,14 +47,22 @@ class Application(dbus.service.Object):
         self.settingsFile = os.path.join(conduit.USER_DIR, "settings.xml")
         self.dbFile = os.path.join(conduit.USER_DIR, "mapping.db")
 
+        #initialize application settings
+        conduit.GLOBALS.settings = Settings()
+
         buildGUI = True
         iconify = False
         whitelist = None
         blacklist = None
         self.ui = "gtk"
+        settings = {}
         try:
-            opts, args = getopt.getopt(sys.argv[1:], "hvs:ciu:w:x:", 
-                ["help", "version", "settings=", "console", "iconify", "ui=", "with-modules=", "without-modules="])
+            opts, args = getopt.getopt(
+                            sys.argv[1:],
+                            "hvf:ciu:w:x:s:",
+                                ("help", "version", "config-file=",
+                                 "console", "iconify", "ui=", "with-modules=",
+                                 "without-modules=", "settings="))
             #parse args
             for o, a in opts:
                 if o in ("-h", "--help"):
@@ -59,7 +71,7 @@ class Application(dbus.service.Object):
                 if o in ("-v", "--version"):
                     print "Conduit %s" % conduit.VERSION
                     sys.exit(0)
-                if o in ("-s", "--settings"):
+                if o in ("-f", "--config-file"):
                      self.settingsFile = os.path.join(os.getcwd(), a)
                 if o in ("-c", "--console"):
                    buildGUI = False
@@ -71,6 +83,13 @@ class Application(dbus.service.Object):
                     whitelist = a.split(",")
                 if o in ("-x", "--without-modules"):
                     blacklist = a.split(",")
+                if o in ("-s", "--settings"):
+                    try:
+                        for i in a.split(','):
+                            k,v = i.split('=')
+                            settings[k] = v
+                    except: pass
+                    conduit.GLOBALS.settings.set_overrides(**settings)
 
         except getopt.GetoptError:
             log.warn("Unknown command line option")
@@ -79,6 +98,8 @@ class Application(dbus.service.Object):
 
         log.info("Conduit v%s Installed: %s" % (conduit.VERSION, conduit.IS_INSTALLED))
         log.info("Python: %s" % sys.version)
+        if settings:
+            log.info("Settings have been overridden: %s" % settings)
         
         #Make conduit single instance. If conduit is already running then
         #make the original process build or show the gui
@@ -168,20 +189,22 @@ class Application(dbus.service.Object):
     def _usage(self):
         print """Usage: conduit [OPTIONS] - Synchronize things
 OPTIONS:
-    -h, --help              Show this message.
-    -c, --console           Launch with no GUI.
-                            (default=no)
-    -s, --settings=FILE     Save settings to FILE.
-                            (default=$XDG_CONFIG_DIR/.conduit/settings.xml)
-    -i, --iconify           Iconify on startup.
-                            (default=no)
-    -u, --ui=NAME           Run with the specified UI
-                            (default=gtk)
-    -w, --with-modules      Only load modules in the named files
-                            (default=load all modules)
-    -x, --without-modules   Do not load modules in the named files
-                            (default=load all modules)
-    -v, --version           Show version information"""
+    -h, --help                  Show this message.
+    -c, --console               Launch with no GUI.
+                                (default=no)
+    -f, --config-file=FILE      Save dataprovider configuration to FILE.
+                                (default=$XDG_CONFIG_DIR/.conduit/settings.xml)
+    -i, --iconify               Iconify on startup.
+                                (default=no)
+    -u, --ui=NAME               Run with the specified UI.
+                                (default=gtk)
+    -w, --with-modules          Only load modules in the named files.
+                                (default=load all modules)
+    -x, --without-modules       Do not load modules in the named files.
+                                (default=load all modules)
+    -s, --settings=key=val,..   Explicitly set internal Conduit settings (keys)
+                                to the given values for this session.
+    -v, --version               Show version information."""
 
     @dbus.service.method(APPLICATION_DBUS_IFACE, in_signature='', out_signature='b')
     def HasGUI(self):
@@ -253,12 +276,18 @@ OPTIONS:
         #unitialize all dataproviders
         log.info("Unitializing dataproviders")
         self.guiSyncSet.quit()
+        log.info("GUI Quit")
         self.dbusSyncSet.quit()
+        log.info("DBus Quit")
 
         #Save the mapping DB
         conduit.GLOBALS.mappingDB.save()
         conduit.GLOBALS.mappingDB.close()
+
+        #Save the application settings
+        conduit.GLOBALS.settings.save()
         
+        log.info("Main Loop Quitting")
         conduit.GLOBALS.mainloop.quit()
 
     @dbus.service.method(APPLICATION_DBUS_IFACE, in_signature='', out_signature='')
