@@ -13,23 +13,21 @@ log = logging.getLogger("dataproviders.DataProvider")
 
 import conduit
 import conduit.ModuleWrapper as ModuleWrapper
-import conduit.Utils as Utils
+import conduit.utils as Utils
 import conduit.Settings as Settings
 
-#Constants used in the sync state machine
-STATUS_NONE = 0
-STATUS_CHANGE_DETECTED = 1
-STATUS_REFRESH = 2
-STATUS_DONE_REFRESH_OK = 3
-STATUS_DONE_REFRESH_ERROR = 4
-STATUS_SYNC = 5
-STATUS_DONE_SYNC_OK = 6
-STATUS_DONE_SYNC_ERROR = 7
-STATUS_DONE_SYNC_SKIPPED = 8
-STATUS_DONE_SYNC_CANCELLED = 9
-STATUS_DONE_SYNC_CONFLICT = 10
-STATUS_DONE_SYNC_NOT_CONFIGURED = 11
-
+STATUS_NONE = _("Ready")
+STATUS_CHANGE_DETECTED = _("New data to sync")
+STATUS_REFRESH = _("Refreshing...")
+STATUS_DONE_REFRESH_OK = _("Refreshed OK")
+STATUS_DONE_REFRESH_ERROR = _("Error Refreshing")
+STATUS_SYNC = _("Synchronizing...")
+STATUS_DONE_SYNC_OK = _("Synchronized OK")
+STATUS_DONE_SYNC_ERROR = _("Error Synchronizing")
+STATUS_DONE_SYNC_SKIPPED = _("Synchronization Skipped")
+STATUS_DONE_SYNC_CANCELLED = _("Synchronization Cancelled")
+STATUS_DONE_SYNC_CONFLICT = _("Synchronization Conflict")
+STATUS_DONE_SYNC_NOT_CONFIGURED = _("Not Configured Correctly")
 
 class DataProviderBase(gobject.GObject):
     """
@@ -46,6 +44,7 @@ class DataProviderBase(gobject.GObject):
     _icon_ = ""
     _module_type_ = "dataprovider"
     _category_ = conduit.dataproviders.CATEGORY_TEST
+    _configurable_ = False
     _out_type_ = ""
     _in_type_ = ""
     
@@ -142,9 +141,6 @@ class DataProviderBase(gobject.GObject):
         """
         Sets the dataprovider status. If the status has changed then emits
         a status-changed signal
-        
-        @param newStatus: The new status
-        @type newStatus: C{int}
         """
         if newStatus != self.get_status():
             self.status = newStatus
@@ -152,44 +148,10 @@ class DataProviderBase(gobject.GObject):
         
     def get_status(self):
         """
-        @returns: The current dataproviders status (code)
-        @rtype: C{int}
+        @returns: The current dataproviders status
         """
         return self.status
         
-    def get_status_text(self):
-        """
-        @returns: a textual representation of the current dataprover status
-        @rtype: C{str}
-        """
-        s = self.get_status()
-        if s == STATUS_NONE:
-            return _("Ready")
-        elif s == STATUS_CHANGE_DETECTED:
-            return _("New data to sync")
-        elif s == STATUS_REFRESH:
-            return _("Refreshing...")
-        elif s == STATUS_DONE_REFRESH_OK:
-            return _("Refreshed OK")
-        elif s == STATUS_DONE_REFRESH_ERROR:
-            return _("Error Refreshing")
-        elif s == STATUS_SYNC:
-            return _("Synchronizing...")
-        elif s == STATUS_DONE_SYNC_OK:
-            return _("Synchronized OK")
-        elif s == STATUS_DONE_SYNC_ERROR:
-            return _("Error Synchronizing")
-        elif s == STATUS_DONE_SYNC_SKIPPED:
-            return _("Synchronization Skipped")
-        elif s == STATUS_DONE_SYNC_CANCELLED:
-            return _("Synchronization Cancelled")
-        elif s == STATUS_DONE_SYNC_CONFLICT:
-            return _("Synchronization Conflict")
-        elif s == STATUS_DONE_SYNC_NOT_CONFIGURED:
-            return _("Not Configured Correctly")
-        else:
-            return "BAD PROGRAMMER"
-            
     def is_busy(self):
         """
         A DataProvider is busy if it is currently in the middle of the
@@ -211,7 +173,7 @@ class DataProviderBase(gobject.GObject):
         Show a configuration box for configuring the dataprovider instance.
         @param window: The parent gtk.Window (to show a modal dialog)
         """
-        log.debug("configure() not overridden by derived class %s" % self._name_)
+        pass
 
     def is_configured(self, isSource, isTwoWay):
         """
@@ -229,7 +191,6 @@ class DataProviderBase(gobject.GObject):
         @returns: Dictionary of strings containing application settings
         @rtype: C{dict(string)}
         """
-        log.debug("get_configuration() not overridden by derived class %s" % self._name_)
         return {}
 
     def get_configuration_xml(self):
@@ -262,12 +223,10 @@ class DataProviderBase(gobject.GObject):
         Restores applications settings
         @param config: dictionary of dataprovider settings to restore
         """
-        log.debug("set_configuration() not overridden by derived class %s" % self._name_)
         for c in config:
             #Perform these checks to stop malformed xml from stomping on
             #unintended variables or posing a security risk by overwriting methods
             if getattr(self, c, None) != None and callable(getattr(self, c, None)) == False:
-                log.debug("Setting %s to %s" % (c, config[c]))
                 setattr(self,c,config[c])
             else:
                 log.warn("Not restoring %s setting: Exists=%s Callable=%s" % (
@@ -300,7 +259,6 @@ class DataProviderBase(gobject.GObject):
                             #fallback to string type
                             log.warn("Cannot convert string (%s) to native type %s\n" % (raw, vtype, traceback.format_exc()))
                             data = str(raw)
-                        log.debug("Read Setting: Name=%s Value=%s Type=%s" % (s.localName, data, type(data)))
                         settings[s.localName] = data
 
             try:
@@ -448,8 +406,8 @@ class DataSink(DataProviderBase):
         @type putData: A L{conduit.DataType.DataType} derived type that this 
         dataprovider is capable of handling
         @param overwrite: If this argument is True, the DP should overwrite
-        an existing datatype instace (if one exists). Generally used in conflict
-        resolution. 
+        an existing datatype instance (if one exists). Generally used in
+        conflict resolution. 
         @type overwrite: C{bool}
         @param LUID: A locally unique identifier representing the location
         where the data was previously put.
@@ -501,9 +459,10 @@ class DataProviderFactory(gobject.GObject):
                     category=category
                     )
         dpw.set_dnd_key(customKey)
-        log.debug("DataProviderFactory %s: Emitting dataprovider-added for %s" % (self, dpw.get_dnd_key()))
+        key = dpw.get_dnd_key()
+        log.debug("DataProviderFactory %s: Emitting dataprovider-added for %s" % (self, key))
         self.emit("dataprovider-added", dpw, klass)
-        return dpw.get_dnd_key()
+        return key
 
     def emit_removed(self, key):
         log.debug("DataProviderFactory %s: Emitting dataprovider-removed for %s" % (self, key))
@@ -518,18 +477,22 @@ class DataProviderFactory(gobject.GObject):
         """
         pass
 
-    def get_configuration_widget(self):
+    def setup_configuration_widget(self):
         """
         If the factory needs to offer configuration options then
-        it should return a gtk.widget here.
+        it should return a gtk.widget here. This widget is then packed
+        into the configuration notebook.
         """
         return None
 
-    def save_configuration(self):
+    def save_configuration(self, ok):
         """
-        If the user closes the configuration panel with RESPONSE_OK (e.g.
-        doesnt click cancel) then this will be called on all derived classes
+        @param ok: True if the user closed the prefs panel with OK, false if 
+        they cancelled it.
         """
         pass
+
+    def get_name(self):
+        return self.__class__.__name__
         
 
