@@ -1,3 +1,4 @@
+import sys
 import os
 import tempfile
 import datetime
@@ -5,11 +6,6 @@ import traceback
 import logging
 log = logging.getLogger("datatypes.File")
 
-try:
-    import gnomevfs
-except ImportError:
-    from gnome import gnomevfs # for maemo
-
 import conduit
 import conduit.datatypes.DataType as DataType
 import conduit.Vfs as Vfs
@@ -32,33 +28,37 @@ class File(DataType.DataType):
           - group: A named group to which this file belongs
         """
         DataType.DataType.__init__(self)
-        #compulsory args
-        self.URI = gnomevfs.URI(URI)
-
+        
+        #you can override the file implmentation at runtime
+        #for testing purposes only
+        implName = kwargs.get("implName", conduit.FILE_IMPL)
+        if implName == "GnomeVfs":
+            import conduit.platform.FileGnomeVfs as FileImpl
+            self.FileImpl = FileImpl
+        elif implName == "GIO":
+            import conduit.platform.FileGio as FileImpl
+            self.FileImpl = FileImpl
+        elif implName == "Python":
+            import conduit.platform.FilePython as FileImpl
+            self.FileImpl = FileImpl
+        else:
+            raise Exception("File Implementation %s Not Supported" % implName)
+            
+        self._file = self.FileImpl.FileImpl(URI)
+        
         #optional args
         self.basePath = kwargs.get("basepath","")
         self.group = kwargs.get("group","")
 
         #instance
-        self.fileInfo = None
-        self.fileExists = False
-        self.triedOpen = False
         self._newFilename = None
         self._newMtime = None
 
         self._isProxyFile = False
         self._proxyFileSize = None
         
-    def _open_file(self):
-        if self.triedOpen == False:
-            self.triedOpen = True
-            self.fileExists = gnomevfs.exists(self.URI)
-
     def _close_file(self):
-        log.debug("Closing file")
-        self.fileInfo = None
-        self.fileExists = False
-        self.triedOpen = False
+        self._file.close()
 
         #check to see if we have applied the rename/mtimes yet
         if self.get_filename() == self._newFilename:
@@ -71,39 +71,9 @@ class File(DataType.DataType):
     def _xfer_check_global_cancel_flag(self):
         return conduit.GLOBALS.cancelled
 
-    def _xfer_progress_callback(self, info, cancel_func):
-        #check if cancelled
-        try:
-            if cancel_func():
-                log.info("Transfer of %s -> %s cancelled" % (info.source_name, info.target_name))
-                return 0
-        except Exception, ex:
-            log.warn("Could not call gnomevfs cancel function")
-            return 0
-        return True
-
     def _get_text_uri(self):
-        """
-        The mixing of text_uri and gnomevfs.URI in the gnomevfs api is very
-        annoying. This function returns the full text uri for the file
-        """
-        return str(self.URI)        
+        return self._file.get_text_uri()
             
-    def _get_file_info(self):
-        """
-        Gets the file info. Because gnomevfs is dumb this method works a lot
-        more reliably than self.vfsFileHandle.get_file_info().
-        
-        Only tries to get the info once for performance reasons
-        """
-        self._open_file()
-        #The get_file_info works more reliably on remote vfs shares
-        if self.fileInfo == None:
-            if self.exists() == True:
-                self.fileInfo = gnomevfs.get_file_info(self.URI, gnomevfs.FILE_INFO_DEFAULT)
-            else:
-                log.warn("Cannot get info on non-existant file %s" % self.URI)
-
     def _defer_rename(self, filename):
         """
         In the event that the file is on a read-only volume this call defers the 
@@ -128,7 +98,8 @@ class File(DataType.DataType):
         
     def _is_tempfile(self):
         tmpdir = tempfile.gettempdir()
-        if self.is_local() and self.URI.path.startswith(tmpdir):
+        path = self._file.get_local_path()
+        if self._file.is_local() and path and path.startswith(tmpdir):
             return True
         else:
             return False
@@ -138,31 +109,26 @@ class File(DataType.DataType):
 
     def _set_file_mtime(self, mtime):
         timestamp = conduit.utils.datetime_get_timestamp(mtime)
-        log.debug("Setting mtime of %s to %s (%s)" % (self.URI, timestamp, type(timestamp)))
-        newInfo = gnomevfs.FileInfo()
-        newInfo.mtime = timestamp
-        gnomevfs.set_file_info(self.URI,newInfo,gnomevfs.SET_FILE_INFO_TIME)
-        #close so the file info is re-read
-        self._close_file()
+        log.debug("Setting mtime of %s to %s (%s)" % (
+                            self._file.get_text_uri(),
+                            timestamp,
+                            type(timestamp)))
+        return self._file.set_mtime(timestamp)
 
     def _set_filename(self, filename):
-        newInfo = gnomevfs.FileInfo()
-        
-        #FIXME: Gnomevfs complains if name is unicode
-        filename =  str(filename)
-        oldname =   str(self.get_filename())
+        oldname = self._file.get_filename()
+        olduri = self._file.get_text_uri()
+        #ignore unicode for equality
+        if str(filename) != str(oldname):
+            newuri = self._file.set_filename(filename)
+            if newuri:
+                log.debug("Rename file %s (%s) -> %s (%s)" % (olduri,oldname,newuri,filename))
+            else:
+                log.debug("Error renaming file %s (%s) -> %s" % (olduri,oldname,filename))
+            return newuri
+        else:
+            return olduri
 
-        if filename != oldname:
-            newInfo.name = filename
-            olduri = self._get_text_uri()
-            newuri = olduri.replace(oldname, filename)
-
-            log.debug("Trying to rename file %s (%s) -> %s (%s)" % (olduri,oldname,newuri,filename))
-            gnomevfs.set_file_info(self.URI,newInfo,gnomevfs.SET_FILE_INFO_NAME)
-            #close so the file info is re-read
-            self.URI = gnomevfs.URI(newuri)
-            self._close_file()
-            
     def set_from_instance(self, f):
         """
         Function to give this file all the properties of the
@@ -170,12 +136,9 @@ class File(DataType.DataType):
         might be pending renames etc on the file that you
         do not want to lose
         """
-        self.URI = f.URI
+        self._file = f._file
         self.basePath = f.basePath
         self.group = f.group
-        self.fileInfo = f.fileInfo
-        self.fileExists = f.fileExists
-        self.triedOpen = f.triedOpen
         self._newFilename = f._newFilename
         self._newMtime = f._newMtime
         self._isProxyFile = f._isProxyFile
@@ -188,7 +151,7 @@ class File(DataType.DataType):
         """
         #Get a temporary file name
         tempname = tempfile.mkstemp(prefix="conduit")[1]
-        log.debug("Tempfile %s -> %s" % (self.URI, tempname))
+        log.debug("Tempfile %s -> %s" % (self._get_text_uri(), tempname))
         filename = self.get_filename()
         mtime = self.get_mtime()
         self.transfer(
@@ -201,8 +164,10 @@ class File(DataType.DataType):
         return tempname
 
     def exists(self):
-        self._open_file()
-        return self.fileExists
+        """
+        Checks the file exists
+        """        
+        return self._file.exists()
 
     def is_local(self):
         """
@@ -210,14 +175,26 @@ class File(DataType.DataType):
         expected that the caller will call get_local_uri, which will
         copy the file to that location, and return the new path
         """
-        return self.URI.is_local
+        return self._file.is_local()
 
     def is_directory(self):
         """
         @returns: True if the File is a directory
         """
-        self._get_file_info()
-        return self.fileInfo.type == gnomevfs.FILE_TYPE_DIRECTORY
+        return self._file.is_directory()
+
+    def make_directory(self):
+        """
+        Makes a directory with the default permissions.
+        """
+        return self._file.make_directory()
+
+    def make_directory_and_parents(self):
+        """
+        Makes a directory and all parents up till the root. Equivilent
+        to mkdir -p
+        """
+        return self._file.make_directory_and_parents()
 
     def force_new_filename(self, filename):
         """
@@ -226,20 +203,8 @@ class File(DataType.DataType):
         if self._is_tempfile() or self._is_proxyfile():
             self._defer_rename(filename)
         else:
-            try:
-                self._set_filename(filename)
-            except gnomevfs.NotSupportedError:
-                #dunno what this is
+            if not self._set_filename(filename):
                 self._defer_rename(filename)
-            except gnomevfs.AccessDeniedError:
-                #file is on readonly filesystem
-                self._defer_rename(filename)
-            except gnomevfs.NotPermittedError:
-                #file is on readonly filesystem
-                self._defer_rename(filename)
-            except gnomevfs.FileExistsError:
-                #I think this is when you rename a file to its current name
-                pass
                 
     def force_new_file_extension(self, ext):
         """
@@ -257,71 +222,35 @@ class File(DataType.DataType):
         if self._is_tempfile() or self._is_proxyfile():
             self._defer_new_mtime(mtime)
         else:
-            try:
-                self._set_file_mtime(mtime)
-            except gnomevfs.NotSupportedError:
-                #dunno what this is
-                self._defer_new_mtime(mtime)
-            except gnomevfs.AccessDeniedError:
-                #file is on readonly filesystem
-                self._defer_new_mtime(mtime)
-            except gnomevfs.NotPermittedError:
-                #file is on readonly filesystem
+            if not self._set_file_mtime(mtime):
                 self._defer_new_mtime(mtime)
 
     def transfer(self, newURIString, overwrite=False, cancel_function=None):
         """
-        Transfers the file to newURI. Thin wrapper around go_gnomevfs_transfer
-        because it also sets the new info of the file. By wrapping the xfer_uri
-        funtion it gives the ability to cancel transfers
+        Transfers the file to newURI. Returning True from 
+        cancel_function gives the ability to cancel transfers
 
         @type newURIString: C{string}
         """
+        trans = self.FileImpl.FileTransferImpl(
+                                source=self._file,
+                                dest=newURIString)
+        
         #the default cancel function just checks conduit.GLOBALS.cancelled
         if cancel_function == None:
             cancel_function = self._xfer_check_global_cancel_flag
 
         if self._is_deferred_rename():
-            newURI = gnomevfs.URI(newURIString)
-            #if it exists and its a directory then transfer into that dir
-            #with the new filename
-            if gnomevfs.exists(newURI):
-                info = gnomevfs.get_file_info(newURI, gnomevfs.FILE_INFO_DEFAULT)
-                if info.type == gnomevfs.FILE_TYPE_DIRECTORY:
-                    #append the new filename
-                    newURI = newURI.append_file_name(self._newFilename)
-                    log.debug("Using deferred filename in transfer")
-        else:
-            newURI = gnomevfs.URI(newURIString)
-            
-        if overwrite:
-            mode = gnomevfs.XFER_OVERWRITE_MODE_REPLACE
-        else:
-            mode = gnomevfs.XFER_OVERWRITE_MODE_SKIP
-        
-        log.debug("Transfering File %s -> %s" % (self.URI, newURI))
+            log.debug("Using deferred filename in transfer")
+            trans.set_destination_filename(self._newFilename)
 
-        #recursively create all parent dirs if needed
-        parent = str(newURI.parent)
-        if not gnomevfs.exists(parent):
-            Vfs.uri_make_directory_and_parents(parent)
-
-        #Copy the file
-        try:        
-            result = gnomevfs.xfer_uri(
-                        source_uri=self.URI,
-                        target_uri=newURI,
-                        xfer_options=gnomevfs.XFER_NEW_UNIQUE_DIRECTORY,
-                        error_mode=gnomevfs.XFER_ERROR_MODE_ABORT,
-                        overwrite_mode=mode,
-                        progress_callback=self._xfer_progress_callback,
-                        data=cancel_function
-                        )
-        except gnomevfs.InterruptedError:
+        #transfer file
+        ok,f = trans.transfer(overwrite, cancel_function)
+        if not ok:
             raise FileTransferError
 
         #close the file and the handle so that the file info is refreshed
-        self.URI = newURI
+        self._file = f
         self._close_file()
         
         #if we have been transferred anywhere (i.e. the destination, our
@@ -335,18 +264,17 @@ class File(DataType.DataType):
             self.force_new_mtime(self._newMtime)
       
     def delete(self):
-        #close the file and the handle so that the file info is refreshed
-        self._close_file()
-        log.debug("Deleting %s" % self.URI)
-        result = gnomevfs.unlink(self.URI)
+        """
+        Deletes the file
+        """
+        log.debug("Deleting %s" % self._file.get_text_uri())
+        self._file.delete()
 
     def get_mimetype(self):
-        self._get_file_info()
-        try:
-            return self.fileInfo.mime_type
-        except ValueError:
-            #Why is gnomevfs so stupid and must I do this for local URIs??
-            return gnomevfs.get_mime_type(self._get_text_uri())
+        """
+        @returns: The file mimetype
+        """
+        return self._file.get_mimetype()
         
     def get_mtime(self):
         """
@@ -359,10 +287,10 @@ class File(DataType.DataType):
         if self._is_deferred_new_mtime():
             return self._newMtime
         else:
-            self._get_file_info()
-            try:
-                return datetime.datetime.fromtimestamp(self.fileInfo.mtime)
-            except:
+            ts = self._file.get_mtime()
+            if ts:
+                return datetime.datetime.fromtimestamp(ts)
+            else:
                 return None
 
     def set_mtime(self, mtime):
@@ -370,10 +298,7 @@ class File(DataType.DataType):
         Sets the modification time of the file
         """
         if mtime != None:
-            try:
-                self.force_new_mtime(mtime)
-            except Exception, err:
-                log.warn("Error setting mtime of %s. \n%s" % (self.URI, traceback.format_exc()))
+            self.force_new_mtime(mtime)
     
     def get_size(self):
         """
@@ -382,11 +307,7 @@ class File(DataType.DataType):
         if self._is_proxyfile():
             return self._proxyFileSize
         else:
-            self._get_file_info()
-            try:
-                return self.fileInfo.size
-            except:
-                return None
+            return self._file.get_size()
 
     def get_hash(self):
         # Join the tags into a string to be hashed so the object is updated if
@@ -402,8 +323,7 @@ class File(DataType.DataType):
         if self._is_deferred_rename():
             return self._newFilename
         else:
-            self._get_file_info()
-            return self.fileInfo.name
+            return self._file.get_filename()
 
     def get_filename_and_extension(self):
         """
@@ -412,7 +332,7 @@ class File(DataType.DataType):
         return os.path.splitext(self.get_filename())
 
     def get_contents_as_text(self):
-        return gnomevfs.read_entire_file(self._get_text_uri())
+        return self._file.get_contents()
         
     def get_local_uri(self):
         """
@@ -422,20 +342,14 @@ class File(DataType.DataType):
         
         If it is a remote file then a local temporary file copy is created
         
-        This function is useful for non gnomevfs enabled libs
-
         @returns: local absolute path the the file or None on error
         @rtype: C{string}
         """
-        if self.is_local():
-            #FIXME: The following call produces a runtime error if the URI
-            #is malformed. Reason number 37 gnomevfs should die
-            u = gnomevfs.get_local_path_from_uri(self._get_text_uri())
-            #Backup approach...
-            #u = self.URI[len("file://"):]
-            return u
-        else:
+        path = self._file.get_local_path()
+        if not path:
             return self.to_tempfile()
+        else:
+            return path
             
     def get_relative_uri(self):
         """
@@ -451,14 +365,14 @@ class File(DataType.DataType):
         Compare me with B based upon their modification times, or optionally
         based on size only
         """
-        if gnomevfs.exists(B.URI) == False:
+        if B.exists() == False:
             return conduit.datatypes.COMPARISON_NEWER
 
         #Compare based on size only?
         if sizeOnly:
             meSize = self.get_size()
             bSize = B.get_size()
-            log.debug("Comparing %s (SIZE: %s) with %s (SIZE: %s)" % (self.URI, meSize, B.URI, bSize))
+            log.debug("Comparing %s (SIZE: %s) with %s (SIZE: %s)" % (self._get_text_uri(), meSize, B._get_text_uri(), bSize))
             if meSize == None or bSize == None:
                 return conduit.datatypes.COMPARISON_UNKNOWN
             elif meSize == bSize:
@@ -469,7 +383,7 @@ class File(DataType.DataType):
         #Else look at the modification times
         meTime = self.get_mtime()
         bTime = B.get_mtime()
-        log.debug("Comparing %s (MTIME: %s) with %s (MTIME: %s)" % (self.URI, meTime, B.URI, bTime))
+        log.debug("Comparing %s (MTIME: %s) with %s (MTIME: %s)" % (self._get_text_uri(), meTime, B._get_text_uri(), bTime))
         if meTime is None:
             return conduit.datatypes.COMPARISON_UNKNOWN
         if bTime is None:            
@@ -485,7 +399,6 @@ class File(DataType.DataType):
         elif meTime == bTime:
             meSize = self.get_size()
             bSize = B.get_size()
-            #log.debug("Comparing %s (SIZE: %s) with %s (SIZE: %s)" % (A.URI, meSize, B.URI, bSize))
             #If the times are equal, and the sizes are equal then assume
             #that they are the same.
             if meSize == None or bSize == None:
@@ -518,35 +431,40 @@ class File(DataType.DataType):
         fd, name = tempfile.mkstemp(prefix="netsync")
         os.write(fd, data['data'])
         os.close(fd)
+
+        implName = conduit.FILE_IMPL
+        if implName == "GnomeVfs":
+            import conduit.platform.FileGnomeVfs as FileImpl
+        elif implName == "GIO":
+            import conduit.platform.FileGio as FileImpl
+        elif implName == "Python":
+            import conduit.platform.FilePython as FileImpl
+        else:
+            raise Exception("File Implementation %s Not Supported" % implName)
         
-        self.URI = gnomevfs.URI(name)
+        self._file = FileImpl.FileImpl(name)
         self.basePath = data['basePath']
         self.group = data['group']
         self._defer_rename(data['filename'])
         self._defer_new_mtime(data['filemtime'])
 
-        #Ensure we re-read the fileInfo
-        self.fileInfo = None
-        self.fileExists = False
-        self.triedOpen = False
-        
         DataType.DataType.__setstate__(self, data)
 
 class TempFile(File):
     """
     Creates a file in the system temp directory with the given contents.
     """
-    def __init__(self, contents=""):
+    def __init__(self, contents, **kwargs):
         #create the file containing contents
         fd, name = tempfile.mkstemp(prefix="conduit")
         os.write(fd, contents)
         os.close(fd)
-        File.__init__(self, name)
+        File.__init__(self, name, **kwargs)
         log.debug("New tempfile created at %s" % name)
         
 class ProxyFile(File):
     """
-    Pretends to be a file for the sake of comparison and transer. Typically
+    Pretends to be a file for the sake of comparison and transfer. Typically
     located on a remote, read only resource, such as http://. Once transferred
     to the local filesystem, it behaves just like a file.
     """
