@@ -1,4 +1,4 @@
-VERSION = "0.3.13.1"
+VERSION = "0.3.14"
 PYTHONDIR = "/usr/local/lib/python2.5/site-packages"
 BIN_DIR = "/usr/local/bin"
 LIB_DIR = "/usr/local/lib"
@@ -10,6 +10,9 @@ SHARED_MODULE_DIR = "/usr/local/lib/conduit/modules"
 # Platform specific implementations
 #
 
+#{GnomeVfs, GIO, Python}
+FILE_IMPL = "GnomeVfs"
+
 #{gtkmozembed, webkit, system}
 BROWSER_IMPL = "gtkmozembed"
 
