@@ -11,11 +11,54 @@ import sys
 import os.path
 import socket
 import datetime
+import urllib2
 import time
 import re
 import logging
 log = logging.getLogger("Utils")
 
+def _get_http_resource(url):
+    try:
+        request = urllib2.Request(url)
+        return urllib2.build_opener().open(request)
+    except urllib2.HTTPError:
+        log.info("URL does not exist: %s" % url)
+        return None
+    except Exception, e:
+        log.warn("Error getting url last modified: %s" % e)
+        return None
+
+def get_http_resource_mimetype(url):
+    """
+    Returns the mimetype of the http resource at the given URL.
+    """
+    a = _get_http_resource(url)
+    if a:
+        return a.headers.getheader('content-type')
+    return None
+
+def get_http_resource_size(url):
+    """
+    Returns the mimetype of the http resource at the given URL.
+    """
+    a = _get_http_resource(url)
+    if a:
+        return int(a.headers.getheader('content-length'))
+    return None
+
+def get_http_resource_last_modified(url):
+    """
+    Returns the last modified date of the http resource at the given URL.
+    @returns: A Float timestamp or None
+    """
+    date = None
+    a = _get_http_resource(url)
+    if a:
+        date = a.headers.getdate('Last-Modified')
+        if date:
+            date = time.mktime(date)
+    return date
+
 def get_proportional_resize(desiredW, desiredH, currentW, currentH):
     """
     Returns proportionally resized co-ordinates for an image
@@ -51,10 +94,21 @@ def program_installed(app):
                 return True
     return False
 
+def ensure_string(arg):
+    """
+    Ensures that arg is str or unicode, returns it as str.
+    """
+    if type(arg) == str:
+        return arg
+    elif type(arg) == unicode:
+        return str(arg)
+    else:
+        raise Exception("must be str or unicode (was %s)" % type(arg))
+
 #
 # Temporary file functions
 #
-def new_tempfile(contents, contentsAreText=True):
+def new_tempfile(contents, **kwargs):
     """
     Returns a new File onject, which has been created in the 
     system temporary directory, and that has been filled with
@@ -63,13 +117,10 @@ def new_tempfile(contents, contentsAreText=True):
     The file is closed when it is returned
     
     @param contents: The data to write into the file
-    @param contentsAreText: Indicates to the OS if the file is text (as opposed
-    to a binary type file
-    @param contentsAreText: C{bool}
     @returns: a L{conduit.datatypes.File}
     """
     import conduit.datatypes.File as File
-    return File.TempFile(contents)
+    return File.TempFile(contents, **kwargs)
 
 def new_tempdir():
     """
